@@ -187,6 +187,30 @@ enum bm_scrollbar_mode {
     BM_SCROLLBAR_LAST,
 };
 
+/**
+ * Password display mode constants for bm_menu instance filter text.
+ *
+ * - @link ::bm_password_mode BM_PASSWORD_NONE @endlink means that filter text displays what the user inputs(default behavior).
+ * - @link ::bm_password_mode BM_PASSWORD_HIDE @endlink means that filter text is not displayed at all.
+ * - @link ::bm_password_mode BM_PASSWORD_INDICATOR @endlink means that filter text is replaced with asterisks.
+ */
+enum bm_password_mode {
+    BM_PASSWORD_NONE,
+    BM_PASSWORD_HIDE,
+    BM_PASSWORD_INDICATOR,
+};
+
+/**
+ * Vertical menu display mode constants for bm_menu instance lines.
+ *
+ * - @link ::bm_lines_mode BM_LINES_DOWN @endlink means that the vertical menu lines will go downwards.
+ * - @link ::bm_lines_mode BM_LINES_UP @endlink means that the vertical menu lines will go upwards
+ */
+enum bm_lines_mode {
+    BM_LINES_DOWN,
+    BM_LINES_UP,
+};
+
 /**
  * Result constants for the menu run functions.
  *
@@ -232,7 +256,8 @@ enum bm_key {
     BM_KEY_LINE_DELETE_LEFT,
     BM_KEY_LINE_DELETE_RIGHT,
     BM_KEY_WORD_DELETE,
-    BM_KEY_PASTE,
+    BM_KEY_PASTE_PRIMARY,
+    BM_KEY_PASTE_CLIPBOARD,
     BM_KEY_TAB,
     BM_KEY_SHIFT_TAB,
     BM_KEY_ESCAPE,
@@ -335,6 +360,8 @@ enum bm_color {
     BM_COLOR_TITLE_FG,
     BM_COLOR_FILTER_BG,
     BM_COLOR_FILTER_FG,
+    BM_COLOR_CURSOR_BG,
+    BM_COLOR_CURSOR_FG,
     BM_COLOR_ITEM_BG,
     BM_COLOR_ITEM_FG,
     BM_COLOR_HIGHLIGHTED_BG,
@@ -343,11 +370,24 @@ enum bm_color {
     BM_COLOR_FEEDBACK_FG,
     BM_COLOR_SELECTED_BG,
     BM_COLOR_SELECTED_FG,
+    BM_COLOR_ALTERNATE_BG,
+    BM_COLOR_ALTERNATE_FG,
     BM_COLOR_SCROLLBAR_BG,
     BM_COLOR_SCROLLBAR_FG,
+    BM_COLOR_BORDER,
     BM_COLOR_LAST
 };
 
+
+/**
+ * Key bindings
+ */
+enum bm_key_binding {
+    BM_KEY_BINDING_DEFAULT,
+    BM_KEY_BINDING_VIM,
+};
+
+
 /**
  * @name Menu Memory
  * @{ */
@@ -476,6 +516,22 @@ BM_PUBLIC void bm_menu_set_lines(struct bm_menu *menu, uint32_t lines);
  */
 BM_PUBLIC uint32_t bm_menu_get_lines(struct bm_menu *menu);
 
+/**
+ * Set the direction in which lines should go after the filter text.
+ *
+ * @param menu bm_menu instance where to set lines direction.
+ * @param mode bm_lines_mode constant.
+ */
+BM_PUBLIC void bm_menu_set_lines_mode(struct bm_menu *menu, enum bm_lines_mode mode);
+
+/**
+ * Get the direction in which lines should go after the filter text.
+ *
+ * @param menu bm_menu instance where to get the lines direction.
+ * @return bm_lines_mode constant
+ */
+BM_PUBLIC enum bm_lines_mode bm_menu_get_lines_mode(struct bm_menu *menu);
+
 /**
  * Set selection wrapping on/off.
  *
@@ -560,6 +616,41 @@ BM_PUBLIC void bm_menu_set_cursor_height(struct bm_menu *menu, uint32_t cursor_h
  */
 BM_PUBLIC uint32_t bm_menu_get_cursor_height(struct bm_menu *menu);
 
+/**
+ * Set width of cursor in pixels.
+ * Some renderers such as ncurses may ignore this when it does not make sense.
+ *
+ * @param menu bm_menu instance where to set cursor height.
+ * @param cursor_width 0 for default cursor height, > 0 for that many pixels.
+ */
+BM_PUBLIC void bm_menu_set_cursor_width(struct bm_menu *menu, uint32_t cursor_width);
+
+/**
+ * Get width of cursor in pixels.
+ *
+ * @param menu bm_menu instance where to get cursor height.
+ * @return uint32_t for max amount of vertical cursors to be shown.
+ */
+BM_PUBLIC uint32_t bm_menu_get_cursor_width(struct bm_menu *menu);
+
+/**
+ * Set horizontal padding in pixels.
+ * Some renderers such as ncurses may ignore this when it does not make sense.
+ *
+ * @param menu bm_menu instance where to set horizontal padding.
+ * @return uint32_t for horizontal padding
+ */
+BM_PUBLIC void bm_menu_set_hpadding(struct bm_menu *menu, uint32_t hpadding);
+
+/**
+ * Get horizontal padding in pixels.
+ * Some renderers such as ncurses may ignore this when it does not make sense.
+ *
+ * @param menu bm_menu instance where to set horizontal padding.
+ * @return uint32_t for horizontal padding
+ */
+BM_PUBLIC uint32_t bm_menu_get_hpadding(struct bm_menu *menu);
+
 /**
  * Get with of menu in pixels.
  *
@@ -576,6 +667,40 @@ BM_PUBLIC uint32_t bm_menu_get_height(struct bm_menu *menu);
  */
 BM_PUBLIC uint32_t bm_menu_get_width(struct bm_menu *menu);
 
+/**
+ * Set the size of the border for the bar
+ *
+ * @param menu bm_menu to get border size from.
+ */
+
+BM_PUBLIC void bm_menu_set_border_size(struct bm_menu* menu, double border_size);
+
+/**
+ * Get the size of the border for the bar
+ *
+ * @param menu bm_menu to get border size from.
+ * @return border size of the menu.
+ */
+
+BM_PUBLIC double bm_menu_get_border_size(struct bm_menu* menu);
+
+/**
+ * Set the radius of the border for the bar
+ *
+ * @param menu bm_menu to get border radius from.
+ */
+
+BM_PUBLIC void bm_menu_set_border_radius(struct bm_menu* menu, double border_radius);
+
+/**
+ * Get the size of the border for the bar
+ *
+ * @param menu bm_menu to get border radius from.
+ * @return border radius of the menu.
+ */
+
+BM_PUBLIC double bm_menu_get_border_radius(struct bm_menu* menu);
+
 /**
  * Set a hexadecimal color for element.
  *
@@ -603,6 +728,24 @@ BM_PUBLIC const char* bm_menu_get_color(const struct bm_menu *menu, enum bm_colo
  */
 BM_PUBLIC void bm_menu_set_scrollbar(struct bm_menu *menu, enum bm_scrollbar_mode mode);
 
+/**
+ * Set fixed height mode of the bar.
+ *
+ * @param menu bm_menu to set the fixed height for.
+ * @param mode to be set.
+ */
+
+BM_PUBLIC void bm_menu_set_fixed_height(struct bm_menu *menu, bool mode);
+
+/**
+ * Get the fixed height mode of the bar.
+ *
+ * @param menu bm_menu to get the fixed height from.
+ * @return current fixed height mode
+ */
+
+BM_PUBLIC bool bm_menu_get_fixed_height(struct bm_menu *menu);
+
 /**
  * Return current scrollbar display mode.
  *
@@ -611,6 +754,42 @@ BM_PUBLIC void bm_menu_set_scrollbar(struct bm_menu *menu, enum bm_scrollbar_mod
  */
 BM_PUBLIC enum bm_scrollbar_mode bm_menu_get_scrollbar(struct bm_menu *menu);
 
+/**
+ * Set the counter display mode.
+ *
+ * @param menu bm_menu to set the counter for.
+ * @param display mode to be set.
+ */
+
+BM_PUBLIC void bm_menu_set_counter(struct bm_menu *menu, bool mode);
+
+/**
+ * Get the counter display mode.
+ *
+ * @param menu bm_menu to get the counter from.
+ * @return current counter display mode
+ */
+
+BM_PUBLIC bool bm_menu_get_counter(struct bm_menu *menu);
+
+/**
+ * Set vim escape exit mode of the bar.
+ *
+ * @param menu bm_menu to set the vim escape exit mode for.
+ * @param mode to be set.
+ */
+
+BM_PUBLIC void bm_menu_set_vim_esc_exits(struct bm_menu *menu, bool mode);
+
+/**
+ * Get the vim escape exit mode of the bar.
+ *
+ * @param menu bm_menu to get the vim escape exit mode from.
+ * @param mode current vim escape exit mode
+ */
+
+BM_PUBLIC bool bm_menu_get_vim_esc_exits(struct bm_menu *menu);
+
 /**
  * Set the vertical alignment of the bar.
  *
@@ -628,6 +807,23 @@ BM_PUBLIC void bm_menu_set_align(struct bm_menu *menu, enum bm_align align);
 
 BM_PUBLIC enum bm_align bm_menu_get_align(struct bm_menu *menu);
 
+/**
+ * Set the vertical/y offset of the window.
+ *
+ * @param menu bm_menu to set offset for.
+ * @param y_offset offset to set.
+ */
+BM_PUBLIC void bm_menu_set_y_offset(struct bm_menu *menu, int32_t y_offset);
+
+/**
+ * Get the vertical/y offset of the window.
+ *
+ * @param menu bm_menu to get y offset from.
+ * @return y offset of the menu.
+ */
+
+BM_PUBLIC int32_t bm_menu_get_y_offset(struct bm_menu *menu);
+
 /**
  * Set the horizontal margin and the relative width factor of the bar.
  *
@@ -706,12 +902,12 @@ BM_PUBLIC bool bm_menu_is_keyboard_grabbed(struct bm_menu *menu);
 BM_PUBLIC void bm_menu_set_panel_overlap(struct bm_menu *menu, bool overlap);
 
 /**
- * Replace input with asterisks.
+ * Set current password mode.
  *
  * @param menu bm_menu instance to set password mode for.
- * @param password true for password mode, false for deafault behavior.
+ * @param password BM_PASSWORD_NONE for default behavior(display what user inputs), BM_PASSWORD_HIDE for no filter text to be displayed, BM_PASSWORD_INDICATOR to replace input with asterisks.
  */
-BM_PUBLIC void bm_menu_set_password(struct bm_menu *menu, bool password);
+BM_PUBLIC void bm_menu_set_password(struct bm_menu *menu, enum bm_password_mode mode);
 
 /**
  * Space entries with title.
@@ -725,9 +921,18 @@ BM_PUBLIC void bm_menu_set_spacing(struct bm_menu *menu, bool spacing);
  * Is password mode activated and input hidden?
  *
  * @param menu bm_menu instance where to get password mode from.
- * @return true if password mode, false otherwise.
+ * @return BM_PASSWORD_NONE is default behavior(display what user inputs), BM_PASSWORD_HIDE is no filter text is being displayed, BM_PASSWORD_INDICATOR is replacing input with asterisks.
  */
-BM_PUBLIC bool bm_menu_get_password(struct bm_menu *menu);
+BM_PUBLIC enum bm_password_mode bm_menu_get_password(struct bm_menu *menu);
+
+
+/**
+ * Specify the key bindings that should be used. 
+ *
+ * @param menu bm_menu instance to set the key binding on.
+ * @param key_binding binding that should be used.
+ */
+BM_PUBLIC void bm_menu_set_key_binding(struct bm_menu *menu, enum bm_key_binding);
 
 
 /**  @} Properties */
@@ -875,7 +1080,7 @@ BM_PUBLIC struct bm_item** bm_menu_get_filtered_items(const struct bm_menu *menu
  *
  * @param menu bm_menu instance to be rendered.
  */
-BM_PUBLIC void bm_menu_render(struct bm_menu *menu);
+BM_PUBLIC bool bm_menu_render(struct bm_menu *menu);
 
 /**
  * Trigger filtering of menu manually.
